function newanalysissettings = analysissettingsPrompt(oldanalysissettings, generalsettings)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

newanalysissettings = oldanalysissettings;

%%%%%%%%%%%%%%%%%%%%%%% Tooltips %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
magnifixedline1 = 'Combined magnification of the objective and any';
magnifixedline2 = 'other optical components such as camera adapters.';
tt.magnifixed = sprintf([magnifixedline1,'\n',magnifixedline2]);
magniautoline1 = 'Magnification can be detected automatically from';
magniautoline2 = 'filename if the name contains the magnification value';
magniautoline3 = 'preceded or followed by an X (case-insensitive)';
magniautoline4 = 'e.g. picture1_20X.tif. More information can be found';
magniautoline5 = 'in the user guide.';
tt.magniauto = sprintf([magniautoline1,'\n',magniautoline2,'\n',magniautoline3,'\n',magniautoline4,'\n',magniautoline5]);
camerapixelline1 = 'Physical pixel size of the camera sensor in micrometers.';
camerapixelline2 = 'The pixel size is typically given in the technical specifications';
camerapixelline3 = 'of the camera manufacturer. Please take into account possible';
camerapixelline4 = 'binning settings. More information can be found in the user guide.';
tt.campixel = sprintf([camerapixelline1,'\n',camerapixelline2,'\n',camerapixelline3,'\n',camerapixelline4]);
imagepixelline1 = 'The theoretical physical pixel size of the image.';
imagepixelline2 = 'Calculated as camera pixel size/magnification.';
tt.impixel = sprintf([imagepixelline1,'\n',imagepixelline2]);

cellsegmentationline1 = 'Segmentation allows cell-by-cell analysis and computation of morphological statistics.';
cellsegmentationline2 = 'Enabling segmentation requires mask image in the same folder as the actual image.';
cellsegmentationline3 = 'Masks are recognized automatically if they include the prefix "segcell_", e.g. segcell_picture1.tif.';
tt.segmentation = sprintf([cellsegmentationline1,'\n',cellsegmentationline2,'\n',cellsegmentationline3]);
channelline1 = 'For RGB images, select the color channel you wish to analyze or "Combined" to merge channels.';
channelline2 = 'For grayscale images, this setting has no effect.';
tt.channel = sprintf([channelline1,'\n',channelline2]);
rotateline1 = 'Orientations computed from the spectrum typically have to be corrected by 90 degrees.';
rotateline2 = 'Some smaller structures of interest can, however, appear at a 90 degree angle relative to';
rotateline3 = 'the major structural complex (e.g. sarcomeres vs. myofilaments). You can take this into account';
rotateline4 = 'by disabling the 90 degree correction for either one or both components.';
tt.rotate = sprintf([rotateline1,'\n',rotateline2,'\n',rotateline3,'\n',rotateline4]);
resoline1 = 'High or maximum resolution settings produce finer spectra with sharper details.';
resoline2 = 'However, high resolution spectra suffer from random noise. Noise can be suppressed';
resoline3 = 'at the expense of lower spectral resolution by selecting the low or minimum noise setting.';
resoline4 = 'This also results in slightly longer computation times. "Balanced" is typically appropriate.';
tt.reso = sprintf([resoline1,'\n',resoline2,'\n',resoline3,'\n',resoline4]);

minwlline1 = 'Set minimum wavelength for spectral analysis.';
minwlline2 = 'This can be used e.g. to exclude small structures';
minwlline3 = 'which are not of interest. The automatic setting';
minwlline4 = 'only limits wavelength based on optical resolution.';
tt.minwl = sprintf([minwlline1,'\n',minwlline2,'\n',minwlline3,'\n',minwlline4]);
maxwlline1 = 'Set maximum wavelength for spectral analysis.';
maxwlline2 = 'This can be used e.g. to exclude large structures';
maxwlline3 = 'which are not of interest. The automatic setting';
maxwlline4 = 'only limits wavelength based on a required angular';
maxwlline5 = 'resolution of 5 degrees.';
tt.maxwl = sprintf([maxwlline1,'\n',maxwlline2,'\n',maxwlline3,'\n',maxwlline4,'\n',maxwlline5]);
exwlline1 = 'Exclude a specific wavelength band from analysis';
exwlline2 = 'e.g. to cut off interfering patterns or artefacts.';
tt.exwl = sprintf([exwlline1,'\n',exwlline2]);

detailguessline1 = 'By default, the detail component is extracted automatically based on';
detailguessline2 = 'signal-to-noise ratios. Semi-automatic detection can be used if the';
detailguessline3 = 'characteristic dimensions of the structures of interest are known or';
detailguessline4 = 'a reasonable guess is available.';
tt.detailguess = sprintf([detailguessline1,'\n',detailguessline2,'\n',detailguessline3,'\n',detailguessline4]);
detailsettingline1 = 'These advanced settings control the detail component detection process';
detailsettingline2 = 'and do not typically need to be modified from their default values.';
detailsettingline3 = 'More information can be found in the user guide.';
tt.detailsetting = sprintf([detailsettingline1,'\n',detailsettingline2,'\n',detailsettingline3]);

%%%%%%%%%%%%%%%%%%%%%%% General components %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
screensize = get(0,'ScreenSize');
defaultBackground = get(0,'defaultUicontrolBackgroundColor');

settingswindow = figure('Name','Analysis settings','NumberTitle','Off','MenuBar','None', ...
    'WindowStyle','modal','CloseRequestFcn',@close_settingswindow,'units','pixels','OuterPosition', ...
    [0.225*screensize(3) 0.05*screensize(4) 0.5*screensize(3) 0.95*screensize(4)],'Color',defaultBackground);
okbutton = uicontrol(settingswindow,'Style','pushbutton','String','OK','Callback',@okbutton_Callback,'units','normalized','position',[0.525 0.01 0.2 0.05]);
cancelbutton = uicontrol(settingswindow,'Style','pushbutton','String','Cancel','Callback',@cancelbutton_Callback,'units','normalized','position',[0.75 0.01 0.2 0.05]);

%%%%%%%%%%%%%%%%%%%%%%% Image box %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
imaginggroup = uibuttongroup('Parent',settingswindow,'Title','Image settings','units','normalized','position',[0.05 0.775 0.9 0.2]);

% Magnification.
magnificationgroup = uibuttongroup('Parent',imaginggroup,'Title','Magnification:','BorderType','etchedin','SelectionChangeFcn',@magnification_Callback,'units','normalized','position',[0.025 0.05 0.3 0.9]);
magnificationmanualbutton = uicontrol(magnificationgroup,'Style','radiobutton','String','Fixed','units','normalized','position',[0.05 0.35 0.9 0.25],'TooltipString',tt.magnifixed);
magnificationautobutton = uicontrol(magnificationgroup,'Style','radiobutton','String','Detect from filename','units','normalized','position',[0.05 0.05 0.9 0.25],'TooltipString',tt.magniauto);
magnificationfield = uicontrol('Parent',magnificationgroup,'Style','edit','Callback',@resolution_Callback,'units','normalized','position',[0.05 0.65 0.3 0.3],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);

% Camera pixel size.
pixelgroup = uibuttongroup('Parent',imaginggroup,'Title','Camera pixel size:','BorderType','etchedin','units','normalized','position',[0.35 0.05 0.275 0.9]);
pixelfield = uicontrol('Parent',pixelgroup,'Style','edit','Callback',@resolution_Callback,'units','normalized','position',[0.05 0.65 0.3 0.3],'HorizontalAlignment','center','BackgroundColor',[1 1 1],'TooltipString',tt.campixel);
pixeltxt = uicontrol('Parent',pixelgroup,'Style','text','String',[char(0181),'m'],'units','normalized','position',[0.4 0.65 0.3 0.3],'HorizontalAlignment','left');
resotxt1 = uicontrol('Parent',pixelgroup,'Style','text','String','Image pixel size (theoretical):','units','normalized','position',[0.05 0.25 0.9 0.3],'HorizontalAlignment','left','TooltipString',tt.impixel);
resolutionfield = uicontrol('Parent',pixelgroup,'Style','text','String','','units','normalized','position',[0.05 0.05 0.3 0.3],'HorizontalAlignment','center','TooltipString',tt.impixel);
resotxt2 = uicontrol('Parent',pixelgroup,'Style','text','String',generalsettings.unitsymbol,'units','normalized','position',[0.4 0.05 0.3 0.3],'HorizontalAlignment','left');

% Segmentation
segmentationgroup = uibuttongroup('Parent',imaginggroup,'Title','Import cell segmentation:','BorderType','etchedin','units','normalized','position',[0.65 0.05 0.325 0.9]);
segmentationoffbutton = uicontrol(segmentationgroup,'Style','radiobutton','String','Disabled','units','normalized','position',[0.05 0.6 0.9 0.25],'TooltipString',tt.segmentation);
segmentationcustombutton = uicontrol(segmentationgroup,'Style','radiobutton','String','Enabled','units','normalized','position',[0.05 0.3 0.9 0.25],'TooltipString',tt.segmentation);

%%%%%%%%%%%%%%%%%%%%%%% Spectral box %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
spectralgroup = uibuttongroup('Parent',settingswindow,'Title','Spectral analysis settings','units','normalized','position',[0.05 0.51 0.9 0.25]);

% Target channel.
targetgroup = uibuttongroup('Parent',spectralgroup,'Title','Target channel:','BorderType','etchedin','units','normalized','position',[0.025 0.6 0.4625 0.35]);
targetchannel = uicontrol(targetgroup,'Style','popupmenu','String','Red|Green|Blue|Combined','units','normalized','position',[0.05 0.3 0.5 0.6],'BackgroundColor',[1 1 1],'TooltipString',tt.channel);

% Orientation rotation.
orientationgroup = uibuttongroup('Parent',spectralgroup,'Title','Rotate orientation by 90 degrees:','units','normalized','position',[0.025 0.05 0.4625 0.5]);
mixedtext = uicontrol(orientationgroup,'Style','text','String','Mixed component:','units','normalized','position',[0.05 0.65 0.5 0.25],'HorizontalAlignment','left','TooltipString',tt.rotate);
detailtext = uicontrol(orientationgroup,'Style','text','String','Detail component:','units','normalized','position',[0.05 0.2 0.5 0.25],'HorizontalAlignment','left','TooltipString',tt.rotate);
mixedtick = uicontrol(orientationgroup,'Style','checkbox','units','normalized','position',[0.575 0.6 0.1 0.3]);
detailtick = uicontrol(orientationgroup,'Style','checkbox','units','normalized','position',[0.575 0.2 0.1 0.3]);

% Spectral resolution.
spectralresogroup = uibuttongroup('Parent',spectralgroup,'Title','Spectral resolution/noise:','units','normalized','position',[0.5125 0.05 0.4625 0.9]);
resobutton5 = uicontrol(spectralresogroup,'Style','radiobutton','String','Maximum resolution','units','normalized','position',[0.05 0.79 0.9 0.18],'TooltipString',tt.reso);
resobutton4 = uicontrol(spectralresogroup,'Style','radiobutton','String','High resolution','units','normalized','position',[0.05 0.60 0.9 0.18],'TooltipString',tt.reso);
resobutton3 = uicontrol(spectralresogroup,'Style','radiobutton','String','Balanced','units','normalized','position',[0.05 0.41 0.9 0.18],'TooltipString',tt.reso);
resobutton2 = uicontrol(spectralresogroup,'Style','radiobutton','String','Low noise','units','normalized','position',[0.05 0.22 0.9 0.18],'TooltipString',tt.reso);
resobutton1 = uicontrol(spectralresogroup,'Style','radiobutton','String','Minimum noise','units','normalized','position',[0.05 0.03 0.9 0.18],'TooltipString',tt.reso);

%%%%%%%%%%%%%%%%%%%%%%% Wavelength box %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
wavelengthgroup = uibuttongroup('Parent',settingswindow,'Title','Wavelength settings','units','normalized','position',[0.05 0.3 0.9 0.2]);

% Lower wavelength limit.
wavelengthlowgroup = uibuttongroup('Parent',wavelengthgroup,'Title','Minimum wavelength:','SelectionChangeFcn',@wavelengthlowgroup_Callback,'units','normalized','position',[0.025 0.05 0.3 0.9]);
wavelengthlowautobutton = uicontrol(wavelengthlowgroup,'Style','radiobutton','String','Automatic','units','normalized','position',[0.05 0.75 0.9 0.2],'TooltipString',tt.minwl);
wavelengthlowmanualbutton = uicontrol(wavelengthlowgroup,'Style','radiobutton','String','Manual','units','normalized','position',[0.05 0.5 0.9 0.2],'TooltipString',tt.minwl);
minimumwlfield = uicontrol(wavelengthlowgroup,'Style','edit','units','normalized','position',[0.05 0.1 0.25 0.3],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);
minimumwltxt = uicontrol('Parent',wavelengthlowgroup,'Style','text','String',generalsettings.unitsymbol,'units','normalized','position',[0.35 0.1 0.3 0.3],'HorizontalAlignment','left');

% Higher wavelength limit.
wavelengthhighgroup = uibuttongroup('Parent',wavelengthgroup,'Title','Maximum wavelength:','SelectionChangeFcn',@wavelengthhighgroup_Callback,'units','normalized','position',[0.35 0.05 0.3 0.9]);
wavelengthhighautobutton = uicontrol(wavelengthhighgroup,'Style','radiobutton','String','Automatic','units','normalized','position',[0.05 0.75 0.9 0.2],'TooltipString',tt.maxwl);
wavelengthhighmanualbutton = uicontrol(wavelengthhighgroup,'Style','radiobutton','String','Manual','units','normalized','position',[0.05 0.5 0.9 0.2],'TooltipString',tt.maxwl);
maximumwlfield = uicontrol(wavelengthhighgroup,'Style','edit','units','normalized','position',[0.05 0.1 0.25 0.3],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);
maximumwltxt = uicontrol('Parent',wavelengthhighgroup,'Style','text','String',generalsettings.unitsymbol,'units','normalized','position',[0.35 0.1 0.3 0.3],'HorizontalAlignment','left');

% Exclusion.
exclusiongroup = uibuttongroup('Parent',wavelengthgroup,'Title','Exclude wavelength band:','SelectionChangeFcn',@exclusiongroup_Callback,'units','normalized','position',[0.675 0.05 0.3 0.9]);
excludeoffbutton = uicontrol(exclusiongroup,'Style','radiobutton','String','Disabled','units','normalized','position',[0.05 0.75 0.9 0.2],'TooltipString',tt.exwl);
excludeonbutton = uicontrol(exclusiongroup,'Style','radiobutton','String','Enabled','units','normalized','position',[0.05 0.5 0.9 0.2],'TooltipString',tt.exwl);
excludetext = uicontrol(exclusiongroup,'Style','text','String','----','units','normalized','position',[0.3 0.05 0.15 0.3],'HorizontalAlignment','center');
excludelowwlfield = uicontrol(exclusiongroup,'Style','edit','units','normalized','position',[0.05 0.1 0.25 0.3],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);
excludehighwlfield = uicontrol(exclusiongroup,'Style','edit','units','normalized','position',[0.45 0.1 0.25 0.3],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);
excludetxt2 = uicontrol('Parent',exclusiongroup,'Style','text','String',generalsettings.unitsymbol,'units','normalized','position',[0.75 0.1 0.2 0.3],'HorizontalAlignment','left');

%%%%%%%%%%%%%%%%%%%%%%%%%%% Detail extraction box %%%%%%%%%%%%%%%%%%%%%%%%%
detailgroup = uibuttongroup('Parent',settingswindow,'Title','Detail detection settings','units','normalized','position',[0.05 0.07 0.9 0.215]);
% Detail component guess.
detailguessgroup = uibuttongroup('Parent',detailgroup,'Title','Wavelength band:','SelectionChangeFcn',@detailguessgroup_Callback,'units','normalized','position',[0.025 0.05 0.3 0.9]);
detailautobutton = uicontrol(detailguessgroup,'Style','radiobutton','String','Automatic','units','normalized','position',[0.05 0.75 0.9 0.2],'TooltipString',tt.detailguess);
detailmanualbutton = uicontrol(detailguessgroup,'Style','radiobutton','String','Semi-automatic','units','normalized','position',[0.05 0.5 0.9 0.2],'TooltipString',tt.detailguess);
detailtext = uicontrol(detailguessgroup,'Style','text','String','----','units','normalized','position',[0.3 0.05 0.15 0.3],'HorizontalAlignment','center');
minguessfield = uicontrol(detailguessgroup,'Style','edit','units','normalized','position',[0.05 0.1 0.25 0.3],'HorizontalAlignment','center','BackgroundColor',[1 1 1],'TooltipString',tt.detailguess);
maxguessfield = uicontrol(detailguessgroup,'Style','edit','units','normalized','position',[0.45 0.1 0.25 0.3],'HorizontalAlignment','center','BackgroundColor',[1 1 1],'TooltipString',tt.detailguess);
detailtxt2 = uicontrol('Parent',detailguessgroup,'Style','text','String',generalsettings.unitsymbol,'units','normalized','position',[0.75 0.1 0.2 0.3],'HorizontalAlignment','left');

% Other detail component settings.
detailothergroup = uibuttongroup('Parent',detailgroup,'Title','Search settings:','SelectionChangeFcn',@detailothergroup_Callback,'units','normalized','position',[0.35 0.05 0.625 0.9]);
detaildefaultbutton = uicontrol(detailothergroup,'Style','radiobutton','String','Default','units','normalized','position',[0.025 0.75 0.2 0.2],'TooltipString',tt.detailsetting);
detailcustombutton = uicontrol(detailothergroup,'Style','radiobutton','String','Custom','units','normalized','position',[0.025 0.5 0.2 0.2],'TooltipString',tt.detailsetting);
% Segment length.
seglengthtxt = uicontrol('Parent',detailothergroup,'Style','text','String','Segment length:','units','normalized','position',[0.25 0.755 0.25 0.225],'HorizontalAlignment','left');
seglengthfield = uicontrol(detailothergroup,'Style','edit','Callback',@detailsettings_Callback,'units','normalized','position',[0.525 0.755 0.075 0.225],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);
% Significance level.
alphatxt = uicontrol('Parent',detailothergroup,'Style','text','String','Significance level:','units','normalized','position',[0.25 0.51 0.25 0.225],'HorizontalAlignment','left');
alphafield = uicontrol(detailothergroup,'Style','edit','Callback',@detailsettings_Callback,'units','normalized','position',[0.525 0.51 0.075 0.225],'HorizontalAlignment','center','BackgroundColor',[1 1 1]); 
% Angle slack.
angleslackhtxt = uicontrol('Parent',detailothergroup,'Style','text','String','Angular tolerance:','units','normalized','position',[0.25 0.265 0.25 0.225],'HorizontalAlignment','left');
angleslackfield = uicontrol(detailothergroup,'Style','edit','Callback',@detailsettings_Callback,'units','normalized','position',[0.525 0.265 0.075 0.225],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);
% Convergence limit.
convergencetxt = uicontrol('Parent',detailothergroup,'Style','text','String','Convergence limit:','units','normalized','position',[0.625 0.755 0.25 0.225],'HorizontalAlignment','left');
convergencefield = uicontrol(detailothergroup,'Style','edit','Callback',@detailsettings_Callback,'units','normalized','position',[0.9 0.755 0.075 0.225],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);    
% Max iterations.
maxitertxt = uicontrol('Parent',detailothergroup,'Style','text','String','Max. iterations:','units','normalized','position',[0.625 0.51 0.25 0.225],'HorizontalAlignment','left');
maxiterfield = uicontrol(detailothergroup,'Style','edit','Callback',@detailsettings_Callback,'units','normalized','position',[0.9 0.51 0.075 0.225],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);    
% Prior width, angle.
sigmaangletxt = uicontrol('Parent',detailothergroup,'Style','text','String','Prior width, angle:','units','normalized','position',[0.625 0.265 0.25 0.225],'HorizontalAlignment','left');
sigmaanglefield = uicontrol(detailothergroup,'Style','edit','Callback',@detailsettings_Callback,'units','normalized','position',[0.9 0.265 0.075 0.225],'HorizontalAlignment','center','BackgroundColor',[1 1 1]);   

%%%%%%%%%%%%%%%%% Set defaults according to old settings %%%%%%%%%%%%%%%%%%
% Pass on the default detail extraction settings, which will never be
% modified.
newanalysissettings.default_segmentlength = oldanalysissettings.default_segmentlength;
newanalysissettings.default_alpha = oldanalysissettings.default_alpha;
newanalysissettings.default_angleslack = oldanalysissettings.default_angleslack;
newanalysissettings.default_convergence_threshold = oldanalysissettings.default_convergence_threshold;
newanalysissettings.default_maxiterations = oldanalysissettings.default_maxiterations;
newanalysissettings.default_sigma_prior_angle = oldanalysissettings.default_sigma_prior_angle;

% Pass on the custom detail extraction settings.
newanalysissettings.segmentlength = oldanalysissettings.segmentlength;
newanalysissettings.alpha = oldanalysissettings.alpha;
newanalysissettings.angleslack = oldanalysissettings.angleslack;
newanalysissettings.convergence_threshold = oldanalysissettings.convergence_threshold;
newanalysissettings.maxiterations = oldanalysissettings.maxiterations;
newanalysissettings.sigma_prior_angle = oldanalysissettings.sigma_prior_angle;

% Magnification and camera settings.
set(magnificationfield,'String',num2str(oldanalysissettings.magnification));
set(pixelfield,'String',num2str(oldanalysissettings.camerapixelsize));
if oldanalysissettings.ismagnificationauto
    set(magnificationgroup,'SelectedObject',magnificationautobutton);
    set(resolutionfield,'String','');
    set(magnificationfield,'Enable','off');
else
    set(magnificationgroup,'SelectedObject',magnificationmanualbutton);
    set(resolutionfield,'String',num2str(oldanalysissettings.imagepixelsize));
    set(magnificationfield,'Enable','on');
end    

% Color channel.
set(targetchannel,'Value',oldanalysissettings.targetchannel);

% Segmentation on/off.
switch oldanalysissettings.issegmentationon
    case 0
        set(segmentationgroup,'SelectedObject',segmentationoffbutton);
    case 1
        set(segmentationgroup,'SelectedObject',segmentationcustombutton);
end

% Spectral resolution/noise parameter.
switch oldanalysissettings.resolutionparameter
    case 1
        set(spectralresogroup,'SelectedObject',resobutton1);
    case 2
        set(spectralresogroup,'SelectedObject',resobutton2);
    case 3
        set(spectralresogroup,'SelectedObject',resobutton3);
    case 4
        set(spectralresogroup,'SelectedObject',resobutton4);
    case 5
        set(spectralresogroup,'SelectedObject',resobutton5);
end

% Orientation correction.
set(mixedtick,'Value',oldanalysissettings.rotatemixed);
set(detailtick,'Value',oldanalysissettings.rotatedetail);

% Low wavelength limit.
if oldanalysissettings.islowlimitauto
    set(wavelengthlowgroup,'SelectedObject',wavelengthlowautobutton);
    set(minimumwlfield,'Enable','off');
else
    set(wavelengthlowgroup,'SelectedObject',wavelengthlowmanualbutton);
    set(minimumwlfield,'Enable','on');
end
if ~isempty(oldanalysissettings.lowlimit) && ~isnan(oldanalysissettings.lowlimit)
    set(minimumwlfield,'String',num2str(oldanalysissettings.lowlimit));
else
    set(minimumwlfield,'String','');
end

% High wavelength limit.
if oldanalysissettings.ishighlimitauto
    set(wavelengthhighgroup,'SelectedObject',wavelengthhighautobutton);
    set(maximumwlfield,'Enable','off');
else
    set(wavelengthhighgroup,'SelectedObject',wavelengthhighmanualbutton);
    set(maximumwlfield,'Enable','on');
end
if ~isempty(oldanalysissettings.highlimit) && ~isnan(oldanalysissettings.highlimit)
    set(maximumwlfield,'String',num2str(oldanalysissettings.highlimit));
else
    set(maximumwlfield,'String','');
end

% Exclusion.
if ~oldanalysissettings.isexcludeon
    set(exclusiongroup,'SelectedObject',excludeoffbutton);
    set(excludelowwlfield,'Enable','off');
    set(excludehighwlfield,'Enable','off');
else
    set(exclusiongroup,'SelectedObject',excludeonbutton);
    set(excludelowwlfield,'Enable','on');
    set(excludehighwlfield,'Enable','on');
end
if ~isempty(oldanalysissettings.excludelow) && ~isnan(oldanalysissettings.excludelow)
    set(excludelowwlfield,'String',num2str(oldanalysissettings.excludelow));
else
    set(excludelowwlfield,'String','');
end
if ~isempty(oldanalysissettings.excludehigh) && ~isnan(oldanalysissettings.excludehigh)
    set(excludehighwlfield,'String',num2str(oldanalysissettings.excludehigh));
else
    set(excludehighwlfield,'String','');
end

% Detail component wavelength range.
if oldanalysissettings.isdetailauto
    set(detailguessgroup,'SelectedObject',detailautobutton);
    set(minguessfield,'Enable','off');
    set(maxguessfield,'Enable','off');
else
    set(detailguessgroup,'SelectedObject',detailmanualbutton);
    set(minguessfield,'Enable','on');
    set(maxguessfield,'Enable','on');
end
if ~isempty(oldanalysissettings.lowguess) && ~isnan(oldanalysissettings.lowguess)
    set(minguessfield,'String',num2str(oldanalysissettings.lowguess));
else
    set(minguessfield,'String','');
end
if ~isempty(oldanalysissettings.highguess) && ~isnan(oldanalysissettings.highguess)
    set(maxguessfield,'String',num2str(oldanalysissettings.highguess));
else
    set(maxguessfield,'String','');
end

% Other detail component settings.
if oldanalysissettings.isdetaildefault
    set(detailothergroup,'SelectedObject',detaildefaultbutton);
    detailothergroup_Callback;
else
    set(detailothergroup,'SelectedObject',detailcustombutton);
    detailothergroup_Callback;
end


%%%%%%%%%%%%%%%%%%%%%%%% Update radio buttons %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Radio button: magnification.
function magnification_Callback(hObject,eventdata)
    selectedbutton = get(magnificationgroup,'SelectedObject');
    if selectedbutton == magnificationautobutton
        set(magnificationfield,'Enable','off');
        set(resolutionfield,'String','');
    elseif selectedbutton == magnificationmanualbutton
        set(magnificationfield,'Enable','on');
        resolution_Callback(hObject,eventdata);
    end
end

% Radio button: low wavelength limit.
function wavelengthlowgroup_Callback(hObject,eventdata)
    selectedbutton = get(wavelengthlowgroup,'SelectedObject');
    if selectedbutton == wavelengthlowautobutton
        set(minimumwlfield,'Enable','off');
    elseif selectedbutton == wavelengthlowmanualbutton
        set(minimumwlfield,'Enable','on');
    end
end

% Radio button: high wavelength limit.
function wavelengthhighgroup_Callback(hObject,eventdata)
    selectedbutton = get(wavelengthhighgroup,'SelectedObject');
    if selectedbutton == wavelengthhighautobutton
        set(maximumwlfield,'Enable','off');
    elseif selectedbutton == wavelengthhighmanualbutton
        set(maximumwlfield,'Enable','on');
    end
end

% Radio button: exclusion.
function exclusiongroup_Callback(hObject,eventdata)
    selectedbutton = get(exclusiongroup,'SelectedObject');
    if selectedbutton == excludeoffbutton
        set(excludelowwlfield,'Enable','off');
        set(excludehighwlfield,'Enable','off');
    elseif selectedbutton == excludeonbutton
        set(excludelowwlfield,'Enable','on');
        set(excludehighwlfield,'Enable','on');
    end
end

% Radio button: detail component wavelength.
function detailguessgroup_Callback(hObject,eventdata)
    selectedbutton = get(detailguessgroup,'SelectedObject');
    if selectedbutton == detailautobutton
        set(minguessfield,'Enable','off');
        set(maxguessfield,'Enable','off');
    elseif selectedbutton == detailmanualbutton
        set(minguessfield,'Enable','on');
        set(maxguessfield,'Enable','on');
    end
end

% Radio button: detail component settings.
function detailothergroup_Callback(hObject,eventdata)
    selectedbutton = get(detailothergroup,'SelectedObject');
    if selectedbutton == detaildefaultbutton
        set(seglengthfield,'Enable','off');
        set(alphafield,'Enable','off');
        set(convergencefield,'Enable','off');
        set(maxiterfield,'Enable','off');
        set(angleslackfield,'Enable','off');
        set(sigmaanglefield,'Enable','off');
        set(seglengthfield,'String',num2str(newanalysissettings.default_segmentlength));
        set(alphafield,'String',num2str(newanalysissettings.default_alpha));
        set(convergencefield,'String',num2str(newanalysissettings.default_convergence_threshold));
        set(maxiterfield,'String',num2str(newanalysissettings.default_maxiterations));
        set(angleslackfield,'String',num2str(newanalysissettings.default_angleslack));
        set(sigmaanglefield,'String',num2str(newanalysissettings.default_sigma_prior_angle));
    elseif selectedbutton == detailcustombutton
        set(seglengthfield,'Enable','on');
        set(alphafield,'Enable','on');
        set(convergencefield,'Enable','on');
        set(maxiterfield,'Enable','on');
        set(angleslackfield,'Enable','on');
        set(sigmaanglefield,'Enable','on');
        if ~isempty(newanalysissettings.segmentlength) && ~isnan(newanalysissettings.segmentlength)
            set(seglengthfield,'String',num2str(newanalysissettings.segmentlength));
        else
            set(seglengthfield,'String','');
        end
        if ~isempty(newanalysissettings.alpha) && ~isnan(newanalysissettings.alpha)
            set(alphafield,'String',num2str(newanalysissettings.alpha));
        else
            set(alphafield,'String','');
        end
        if ~isempty(newanalysissettings.convergence_threshold) && ~isnan(newanalysissettings.convergence_threshold)
            set(convergencefield,'String',num2str(newanalysissettings.convergence_threshold));
        else
            set(convergencefield,'String','');
        end
        if ~isempty(newanalysissettings.maxiterations) && ~isnan(newanalysissettings.maxiterations)
            set(maxiterfield,'String',num2str(newanalysissettings.maxiterations));
        else
            set(maxiterfield,'String','');
        end
        if ~isempty(newanalysissettings.angleslack) && ~isnan(newanalysissettings.angleslack)
            set(angleslackfield,'String',num2str(newanalysissettings.angleslack));
        else
            set(angleslackfield,'String','');
        end
        if ~isempty(newanalysissettings.sigma_prior_angle) && ~isnan(newanalysissettings.sigma_prior_angle)
            set(sigmaanglefield,'String',num2str(newanalysissettings.sigma_prior_angle));
        else
            set(sigmaanglefield,'String','');
        end
    end
end

% Update additional detail extraction settings.
function detailsettings_Callback(hObject,eventdata)
    newanalysissettings.segmentlength = str2double(get(seglengthfield,'String'));
    newanalysissettings.alpha = str2double(get(alphafield,'String'));
    newanalysissettings.angleslack = str2double(get(angleslackfield,'String'));
    newanalysissettings.convergence_threshold = str2double(get(convergencefield,'String'));
    newanalysissettings.maxiterations = str2double(get(maxiterfield,'String'));
    newanalysissettings.sigma_prior_angle = str2double(get(sigmaanglefield,'String'));
end

% Update resolution.
function resolution_Callback(hObject,eventadata)
   newanalysissettings.magnification = abs(str2double(get(magnificationfield,'String')));
   if isnan(newanalysissettings.magnification) || newanalysissettings.magnification == 0 || newanalysissettings.magnification == Inf
        errorbox = errordlg('Invalid magnification value!','Invalid parameter','modal');
        uiwait(errorbox);
        set(magnificationfield,'String',num2str(oldanalysissettings.magnification));
        newanalysissettings.magnification = oldanalysissettings.magnification;
   end
   newanalysissettings.camerapixelsize = abs(str2double(get(pixelfield,'String')));
   if isnan(newanalysissettings.camerapixelsize) || newanalysissettings.camerapixelsize == 0 || newanalysissettings.camerapixelsize == Inf
        errorbox = errordlg('Invalid pixel size value!','Invalid parameter','modal');
        uiwait(errorbox);
        set(pixelfield,'String',num2str(oldanalysissettings.camerapixelsize));
        newanalysissettings.camerapixelsize = oldanalysissettings.camerapixelsize;
   end           
   newanalysissettings.imagepixelsize = generalsettings.dimensionfactor*newanalysissettings.camerapixelsize/newanalysissettings.magnification;
   set(resolutionfield,'String',num2str(newanalysissettings.imagepixelsize));
end


%%%%%%%%%%%%%%%%%%%%%%%% Check settings if user pressed ok %%%%%%%%%%%%%%%%
function okbutton_Callback(hObject,eventdata)
    % Color channels and segmentation.
    newanalysissettings.targetchannel = get(targetchannel,'Value');
    selectedbutton = get(segmentationgroup,'SelectedObject');
    if selectedbutton == segmentationoffbutton
        newanalysissettings.issegmentationon = 0;
    elseif selectedbutton == segmentationcustombutton
        newanalysissettings.issegmentationon = 1;
    end

    % Magnification.
    selectedbutton = get(magnificationgroup,'SelectedObject');
    if selectedbutton == magnificationautobutton
        newanalysissettings.ismagnificationauto = 1;
    else
        newanalysissettings.ismagnificationauto = 0;
    end
    newanalysissettings.magnification = str2double(get(magnificationfield,'String'));
    if isnan(newanalysissettings.magnification) || newanalysissettings.magnification <= 0 || newanalysissettings.magnification == Inf
        errorbox = errordlg('Invalid magnification value!','Invalid parameter','modal');
        uiwait(errorbox);
        set(magnificationfield,'String',num2str(oldanalysissettings.magnification));
        newanalysissettings.magnification = oldanalysissettings.magnification;
        return;
    end

    % Camera pixel size.
    newanalysissettings.camerapixelsize = str2double(get(pixelfield,'String'));
    if isnan(newanalysissettings.camerapixelsize) || newanalysissettings.camerapixelsize <= 0 || newanalysissettings.camerapixelsize == Inf
        errorbox = errordlg('Invalid pixel size value!','Invalid parameter','modal');
        uiwait(errorbox);
        set(pixelfield,'String',num2str(oldanalysissettings.camerapixelsize));
        newanalysissettings.camerapixelsize = oldanalysissettings.camerapixelsize;
        return;
    end           
    newanalysissettings.imagepixelsize = generalsettings.dimensionfactor*newanalysissettings.camerapixelsize/newanalysissettings.magnification;
    newanalysissettings.resolution = 1/newanalysissettings.imagepixelsize;

    % Orientation correction.
    newanalysissettings.rotatemixed = get(mixedtick,'Value');
    newanalysissettings.rotatedetail = get(detailtick,'Value');

    % Spectral resolution/noise parameter.
    selectedbutton = get(spectralresogroup,'SelectedObject');
    switch selectedbutton
        case resobutton1
            newanalysissettings.resolutionparameter = 1;
        case resobutton2
            newanalysissettings.resolutionparameter = 2;
        case resobutton3
            newanalysissettings.resolutionparameter = 3;
        case resobutton4
            newanalysissettings.resolutionparameter = 4;
        case resobutton5
            newanalysissettings.resolutionparameter = 5;
    end

    % Wavelength limits.
    selectedbutton = get(wavelengthlowgroup,'SelectedObject');
    if selectedbutton == wavelengthlowautobutton
        newanalysissettings.islowlimitauto = 1;
    else
        newanalysissettings.islowlimitauto = 0;
    end
    selectedbutton = get(wavelengthhighgroup,'SelectedObject');
    if selectedbutton == wavelengthhighautobutton
        newanalysissettings.ishighlimitauto = 1;
    else
        newanalysissettings.ishighlimitauto = 0;
    end
    newanalysissettings.lowlimit = abs(str2double(get(minimumwlfield,'String')));
    newanalysissettings.highlimit = abs(str2double(get(maximumwlfield,'String')));
    % Check for valid inputs.
    if ~newanalysissettings.islowlimitauto && isnan(newanalysissettings.lowlimit)
        errorbox = errordlg('Invalid minimum wavelength!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    if ~newanalysissettings.ishighlimitauto && isnan(newanalysissettings.highlimit)
        errorbox = errordlg('Invalid maximum wavelength!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Lower limit has to be smaller than higher limit, if both were
    % given by the user.
    if (~newanalysissettings.islowlimitauto && ~newanalysissettings.ishighlimitauto) && (newanalysissettings.lowlimit >= newanalysissettings.highlimit)
        errorbox = errordlg('Minimum wavelength has to be lower than maximum wavelength!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Lower limit has to be larger than 2*resolution, if magnification
    % was given by the user.
    if ~newanalysissettings.ismagnificationauto && ~newanalysissettings.islowlimitauto && (newanalysissettings.lowlimit < 2*newanalysissettings.imagepixelsize)
        errorbox = errordlg('Minimum wavelength is limited by the sampling theorem and has been set equal to resolution doubled!','Invalid parameter','modal');
        uiwait(errorbox);
        set(minimumwlfield,'String',num2str(2*newanalysissettings.imagepixelsize));
        return;
    end    

    % Exclusion.
    selectedbutton = get(exclusiongroup,'SelectedObject');
    if selectedbutton == excludeonbutton
        newanalysissettings.isexcludeon = 1;
    elseif selectedbutton == excludeoffbutton
        newanalysissettings.isexcludeon = 0;
    end
    newanalysissettings.excludelow = abs(str2double(get(excludelowwlfield,'String')));
    newanalysissettings.excludehigh = abs(str2double(get(excludehighwlfield,'String')));
    % Check for valid inputs.
    if newanalysissettings.isexcludeon && (isnan(newanalysissettings.excludelow) || isnan(newanalysissettings.excludehigh))
        errorbox = errordlg('Invalid excluded wavelengths!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Lower exclusion limit has to be smaller than higher exclusion limit.
    if newanalysissettings.isexcludeon && (newanalysissettings.excludelow >= newanalysissettings.excludehigh)
        errorbox = errordlg('Minimum wavelength to exclude has to be lower than maximum wavelength to exclude!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Exclusion limits should be within the wavelength range, if the
    % range is manually given.
    if newanalysissettings.isexcludeon && ~newanalysissettings.islowlimitauto && (newanalysissettings.excludelow < newanalysissettings.lowlimit)
        errorbox = errordlg('Wavelengths to exclude should be within the wavelength range!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    if newanalysissettings.isexcludeon && ~newanalysissettings.ishighlimitauto && (newanalysissettings.excludehigh > newanalysissettings.highlimit)
        errorbox = errordlg('Wavelengths to exclude should be within the wavelength range!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Lower exclusion limit still has to be equal to or larger than
    % 2*resolution, if magnification was given by the user.
    if newanalysissettings.isexcludeon && ~newanalysissettings.ismagnificationauto && (newanalysissettings.excludelow < 2*newanalysissettings.imagepixelsize)
        errorbox = errordlg('Minimum wavelength to exclude is limited by the sampling theorem and has been set equal to resolution doubled!','Invalid parameter','modal');
        uiwait(errorbox);
        set(excludelowwlfield,'String',num2str(2*newanalysissettings.imagepixelsize));
        return;
    end    

    % Detail component wavelength range.
    selectedbutton = get(detailguessgroup,'SelectedObject');
    if selectedbutton == detailautobutton
        newanalysissettings.isdetailauto = 1;
    else
        newanalysissettings.isdetailauto = 0;
    end
    newanalysissettings.lowguess = abs(str2double(get(minguessfield,'String')));
    newanalysissettings.highguess = abs(str2double(get(maxguessfield,'String')));
    % Check for valid inputs.
    if ~newanalysissettings.isdetailauto && (isnan(newanalysissettings.lowguess) || isnan(newanalysissettings.highguess))
        errorbox = errordlg('Invalid detail component wavelengths!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Lower guess has to be smaller than higher guess.
    if ~newanalysissettings.isdetailauto && (newanalysissettings.lowguess >= newanalysissettings.highguess)
        errorbox = errordlg('Lower guess for detail component wavelength must be lower than higher guess!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Guesses should be within the wavelength range, if the range is manually given.
    if ~newanalysissettings.isdetailauto && ~newanalysissettings.islowlimitauto && (newanalysissettings.lowguess < newanalysissettings.lowlimit)
        errorbox = errordlg('Guesses for detail component wavelength should be within the wavelength range!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    if ~newanalysissettings.isdetailauto && ~newanalysissettings.ishighlimitauto && (newanalysissettings.highguess > newanalysissettings.highlimit)
        errorbox = errordlg('Guesses for detail component wavelength should be within the wavelength range!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Lower guess still has to be equal to or larger than 2*resolution,
    % if magnification was given by the user.
    if ~newanalysissettings.isdetailauto && ~newanalysissettings.ismagnificationauto && (newanalysissettings.lowguess < 2*newanalysissettings.imagepixelsize)
        errorbox = errordlg('Lower guess for detail component wavelength is limited by the sampling theorem and has been set equal to resolution doubled!','Invalid parameter','modal');
        uiwait(errorbox);
        set(minguessfield,'String',num2str(2*newanalysissettings.imagepixelsize));
        return;
    end
    % If the detail component and excluded range overlap, warn the
    % user.
    if ~newanalysissettings.isdetailauto && newanalysissettings.isexcludeon && ...
            ((newanalysissettings.lowguess >= newanalysissettings.excludelow && newanalysissettings.lowguess <= newanalysissettings.excludehigh) || ...
            (newanalysissettings.highguess >= newanalysissettings.excludelow && newanalysissettings.highguess <= newanalysissettings.excludehigh))
        errorbox = warndlg('Guesses for detail component wavelength range overlap with the excluded wavelength range!','Warning','modal');
        uiwait(errorbox);
    end 

    % Other detail component settings.
    selectedbutton = get(detailothergroup,'SelectedObject');
    if selectedbutton == detaildefaultbutton
        newanalysissettings.isdetaildefault = 1;
    else
        newanalysissettings.isdetaildefault = 0;
    end 
    % Check for valid inputs.
    % Segment length has to be between zero (noninclusive) and one (noninclusive).
    if ~newanalysissettings.isdetaildefault && (newanalysissettings.segmentlength <= 0 || newanalysissettings.segmentlength >= 1)
        errorbox = errordlg('Segment length has to be between zero and one (non-inclusive)!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Significance level has to be between zero (noninclusive) and one (noninclusive).
    if ~newanalysissettings.isdetaildefault && (newanalysissettings.alpha <= 0 || newanalysissettings.alpha >= 1)
        errorbox = errordlg('Significance level has to be between zero and one (non-inclusive)!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Angular tolerance has to be between 0 and 90.
    if ~newanalysissettings.isdetaildefault && (newanalysissettings.angleslack < 0 || newanalysissettings.angleslack >= 90)
        errorbox = errordlg('Angular tolerance has to be between zero (inclusive) and 90 (non-inclusive) degrees!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Convergence limit has to be positive or zero.
    if ~newanalysissettings.isdetaildefault && newanalysissettings.convergence_threshold < 0
        errorbox = errordlg('Convergence limit cannot be negative!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Max iterations has to be positive or zero.
    if ~newanalysissettings.isdetaildefault && newanalysissettings.maxiterations < 0
        errorbox = errordlg('Maximum number of iterations cannot be negative!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end
    % Prior width, angle, has to be larger than zero.
    if ~newanalysissettings.isdetaildefault && newanalysissettings.sigma_prior_angle <= 0
        errorbox = errordlg('Prior width (angle) has to be larger than zero!','Invalid parameter','modal');
        uiwait(errorbox);
        return;
    end

    close_settingswindow;
end

%%%%%%%%%%%%%%%%%%%%% Retrieve old settings if user pressed cancel %%%%%%%%
function cancelbutton_Callback(hObject,eventdata)
    newanalysissettings = oldanalysissettings;
    close_settingswindow;
end

%%%%%%%%%%%%%%% Close the settings window if requested or wait for it %%%%%
function close_settingswindow(hObject,eventdata)
    delete(settingswindow);
end
waitfor(settingswindow);

end

