function magnification = detectMagnification(filename)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% First try the format <magnification>x.
% Try to find patterns starting with a digit 1-9, followed by any number of
% digits 0-9, possibly separated by decimal points, followed by X or x,
% followed by a non-numeric, non-alphabetic character.
detectedpattern = regexpi(filename,'[1-9]\d*(\.\d*)?(?=X[^A-Z0-9])','match');

% If the pattern was not found, try the format x<magnification>.
% Try to find patterns starting with a non-numeric, non-alphabetic
% character, followed by X or x, followed by a digit 1-9,
% followed by any number of digits 0-9, possibly separated by decimal points.
if isempty(detectedpattern)
    detectedpattern = regexpi(filename,'(?<=[^A-Z0-9]X)?[1-9]\d*(\.\d*)?','match');
end

% If a suitable substring was not found or multiple candidates were found,
% magnification cannot be deduced from the filename.
if length(detectedpattern) ~= 1
    magnification = [];
else
% Otherwise convert to double.
    magnification = str2double(detectedpattern{1});
end