function exportsuccess = exportResultsRaw(allfilenames,shortfilenames,temppath,filetype,unitsymbol)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

exportsuccess = 1;

try
    % Ask for output file.
    if strcmpi(filetype,'raw')
        [filename,pathname] = uiputfile('*.txt','Save as','analysisresult.txt');
        if filename == 0
            return;
        else
            outputfile = [pathname,filename];
        end
        % Mu might cause incompatibility.
        if strcmp(unitsymbol,[char(0181),'m'])
            unitsymbol = 'um';
        end
    elseif strcmpi(filetype,'figures')
        [filename,pathname] = uiputfile('*','Save as','analysisresult');
        if filename == 0
            return;
        else
            outputfile = [pathname,filename];
        end
    else
        disp('Unkown filetype. Filetype must be spreadsheet, text, raw or figures.');
        return;
    end

    % Create waitbox.
    screensize = get(0,'ScreenSize');
    waitdialog = dialog('units','pixels','OuterPosition',[0.42*screensize(3) 0.45*screensize(4) 0.16*screensize(3) 0.1*screensize(4)], ...
                        'WindowStyle','modal','Visible','On','ButtonDownFcn','','CloseRequestFcn','');
    waittext = uicontrol(waitdialog,'Style','text','String','Please wait...','units','normalized','position',[0.1 0.2 0.8 0.6],'HorizontalAlignment','center');
    drawnow;

    headerformat = '%s\t%u\t%s\r\n';

    emptyraw = 0;
    for imageind=1:length(allfilenames)
        set(waittext,'String',['Processing data of image ',num2str(imageind),'/',num2str(length(allfilenames)),'...']);
        drawnow;

        % Check if results exist for this image. If not, skip image.
        % Otherwise, load the data.
        if strcmpi(temppath,'sameasimages') && exist([allfilenames{imageind},'.temp.MAT'],'file')
            temp = load([allfilenames{imageind},'.temp.MAT']);
            result = temp.result;
            clear temp;
        elseif ~strcmpi(temppath,'sameasimages') && exist([temppath,filesep,strrep(strrep(allfilenames{imageind},filesep,'_'),':','_'),'.temp.MAT'],'file') 
            temp = load([temppath,filesep,strrep(strrep(allfilenames{imageind},filesep,'_'),':','_'),'.temp.MAT']);
            result = temp.result;
            clear temp;
        else
            continue;
        end

        % If raw data is requested but there is no spectral data, skip to next
        % image.
        if isempty(result(1).fibrildata)
            emptyraw = emptyraw + 1;
            continue;
        end

        % Write raw data.
        if strcmpi(filetype,'raw')
            % Open output stream.
            if imageind == 1
                outputstream = fopen(outputfile,'w');
            else
                outputstream = fopen(outputfile,'a');
            end

            for cellind=1:length(result)
                % Data formats for orientation and distance distributions.
                numofpointsorientation = length(result(cellind).fibrildata.orientation_mixed.distribution);
                numofpointsdistance = length(result(cellind).fibrildata.distance_mixed.distribution);
                dataformatorientation = [repmat('%f\t',1,numofpointsorientation-1),'%f\r\n'];
                dataformatdistance = [repmat('%f\t',1,numofpointsdistance-1),'%f\r\n'];

                headerlines = {allfilenames{imageind} result(cellind).cellids 'Mixed_orientation_values'; ...
                              allfilenames{imageind} result(cellind).cellids 'Mixed_orientation_index (radian)'; ...
                              allfilenames{imageind} result(cellind).cellids 'Detail_orientation_values'; ...
                              allfilenames{imageind} result(cellind).cellids 'Detail_orientation_index (radian)'; ...
                              allfilenames{imageind} result(cellind).cellids 'Mixed_wavelength_values'; ...
                              allfilenames{imageind} result(cellind).cellids ['Mixed_wavelength_index (',unitsymbol,')']; ...
                              allfilenames{imageind} result(cellind).cellids 'Detail_wavelength_values'; ...
                              allfilenames{imageind} result(cellind).cellids ['Detail_wavelength_index (',unitsymbol,')']};

                datalines = {result(cellind).fibrildata.orientation_mixed.distribution'; ...
                            result(cellind).fibrildata.orientation_mixed.angles'; ...
                            result(cellind).fibrildata.orientation_detail.distribution'; ...
                            result(cellind).fibrildata.orientation_detail.angles'; ...
                            result(cellind).fibrildata.distance_mixed.distribution; ...
                            result(cellind).fibrildata.distance_mixed.polar; ...
                            result(cellind).fibrildata.distance_detail.distribution; ...
                            result(cellind).fibrildata.distance_detail.polar};

                for rowind=1:size(datalines,1)
                    % Write header.
                    exportsuccess = fprintf(outputstream,headerformat,headerlines{rowind,:});
                    % Write orientation distribution data.
                    if rowind < 5
                        exportsuccess = fprintf(outputstream,dataformatorientation,datalines{rowind,:});
                    % Write wavelength distribution data.
                    else
                        exportsuccess = fprintf(outputstream,dataformatdistance,datalines{rowind,:});
                    end
                end
            end

            fclose(outputstream);
            if exportsuccess == 0
                break;
            end

        % Output figures.
        elseif strcmpi(filetype,'figures')
            for cellind=1:length(result)
                outputfig = figure('Visible','Off');
                outputax = axes('Parent',outputfig);
                % Mixed component.
                plotOrientationPolar(outputax,result(cellind).fibrildata.orientation_mixed);
                print(outputfig,'-dtiff',[outputfile,'_',shortfilenames{imageind},'_',num2str(result(cellind).cellids),'_orientation_mixed.tif']);
                plotWavelengthDistribution(outputax,result(cellind).fibrildata.distance_mixed,unitsymbol);
                print(outputfig,'-dtiff',[outputfile,'_',shortfilenames{imageind},'_',num2str(result(cellind).cellids),'_wavelength_mixed.tif']);                       
                % Detail component.
                plotOrientationPolar(outputax,result(cellind).fibrildata.orientation_detail);
                print(outputfig,'-dtiff',[outputfile,'_',shortfilenames{imageind},'_',num2str(result(cellind).cellids),'_orientation_detail.tif']);
                plotWavelengthDistribution(outputax,result(cellind).fibrildata.distance_detail,unitsymbol);
                print(outputfig,'-dtiff',[outputfile,'_',shortfilenames{imageind},'_',num2str(result(cellind).cellids),'_wavelength_detail.tif']);            
            end
            close(outputfig);
        end
    end

    % Kill waitbox.
    if emptyraw >= length(allfilenames)
        set(waittext,'String','No data to export!');
    else
        set(waittext,'String','Finished exporting data');
    end
    pause(1);
    set(waitdialog,'CloseRequestFcn','closereq');
    close(waitdialog);
    % Convert number of written bytes to 1 if write was succesfull.
    if exportsuccess > 0
        exportsuccess = 1;
    end
    
catch
    if ishandle(waitdialog)
        set(waitdialog,'CloseRequestFcn','closereq');
        close(waitdialog);
    end
end