function exportsuccess = exportResultsSummary(allfilenames,shortfilenames,temppath,filetype,unitsymbol)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

exportsuccess = 1;

try
    % Ask for output file.
    if strcmpi(filetype,'spreadsheet')
        [filename,pathname] = uiputfile('*.xls','Save as','analysisresult.xls');
        if filename == 0
            return;
        else
            outputfile = [pathname,filename];
        end
    elseif strcmpi(filetype,'text')
        [filename,pathname] = uiputfile('*.txt','Save as','analysisresult.txt');
        if filename == 0
            return;
        else
            outputfile = [pathname,filename];
        end
    else
        disp('Unkown filetype. Filetype must be spreadsheet, text, raw or figures.');
        return;
    end

    % Create waitbox.
    screensize = get(0,'ScreenSize');
    waitdialog = dialog('units','pixels','OuterPosition',[0.42*screensize(3) 0.45*screensize(4) 0.16*screensize(3) 0.1*screensize(4)], ...
                        'WindowStyle','modal','Visible','On','ButtonDownFcn','','CloseRequestFcn','');
    waittext = uicontrol(waitdialog,'Style','text','String','Please wait...','units','normalized','position',[0.1 0.2 0.8 0.6],'HorizontalAlignment','center');
    drawnow;

    % Headers for .xls files.
    xlsheadercell = {'Center X (pixels)','Center Y (pixels)',['Center X (',unitsymbol,')'],['Center Y (',unitsymbol,')'],['Area (',unitsymbol,'^2)'],['Perimeter (',unitsymbol,')'],['Major axis (',unitsymbol,')'],['Minor axis (',unitsymbol,')'],'Eccentricity','Solidity'};
    xlsheaderori = {'Mean orientation','Circ. variance','Angular std','Circ. skewness','Circ. kurtosis'};
    xlsheaderwl = {['Mean wavelength(',unitsymbol,')'],['Median wavelength (',unitsymbol,')'],['Mode wavelength (',unitsymbol,')'],['Std wavelength(',unitsymbol,')']};
    xlsheaderdata = ['Image' 'Full path' 'Cell' 'Component' xlsheaderori xlsheaderwl 'Component' xlsheaderori xlsheaderwl 'Detail SNR (dB)' xlsheadercell];
    xlsheaderinfo = {'Image','Full path','Magnification','Camera pixel (um)','Segmentation','Target ch','Rotate mixed','Rotate detail','Spectral resolution',['WL low (',unitsymbol,')'],['WL high (',unitsymbol,')'],['Exclude low (',unitsymbol,')'],['Exclude high (',unitsymbol,')'],['Detail low (',unitsymbol,')'],['Detail high (',unitsymbol,')'],'Detail search','Segment length','Significance level','Angular tolerance','Convergence limit','Max. iterations','Prior width'};
    cellcols = length(xlsheadercell);
    oricols = length(xlsheaderori);
    wlcols = length(xlsheaderwl);
    oriwlcols = oricols + wlcols;

    % Loop through all images and parse the data structure for export.
    emptyraw = 0;
    datatowrite = {};
    infotowrite = {};

    for imageind=1:length(allfilenames)
        set(waittext,'String',['Processing data of image ',num2str(imageind),'/',num2str(length(allfilenames)),'...']);
        drawnow;
        % Check if results exist for this image. If not, skip image.
        % Otherwise, load the data.
        if strcmpi(temppath,'sameasimages') && exist([allfilenames{imageind},'.temp.MAT'],'file')
            temp = load([allfilenames{imageind},'.temp.MAT']);
            result = temp.result;
            clear temp;
        elseif ~strcmpi(temppath,'sameasimages') && exist([temppath,filesep,strrep(strrep(allfilenames{imageind},filesep,'_'),':','_'),'.temp.MAT'],'file') 
            temp = load([temppath,filesep,strrep(strrep(allfilenames{imageind},filesep,'_'),':','_'),'.temp.MAT']);
            result = temp.result;
            clear temp;
        else
            continue;
        end
        % If there are no results in the file for this image, skip to next image.
        if isempty(result(1).fibrildata)
            emptyraw = emptyraw + 1;
            continue;
        end

        % Get the settings used for this image.
        sets = result(1).analysissettings;
        % Segmentation.
        switch sets.issegmentationon
            case 0
                seg = 'Disabled';
            case 1
                seg = 'Enabled';
        end
        % Target channel.
        switch sets.targetchannel
            case 1
                targetch = 'Red';
            case 2
                targetch = 'Green';
            case 3
                targetch = 'Blue';
            case 4
                targetch = 'Combined';
        end
        % Component rotation.
        switch sets.rotatemixed
            case 0
                rotmixed = 'No';
            case 1
                rotmixed = 'Yes';
        end
        switch sets.rotatedetail
            case 0
                rotdetail = 'No';
            case 1
                rotdetail = 'Yes';
        end
        % Spectral resolution parameter.
        switch sets.resolutionparameter
            case 1
                reso = 'Minimum noise';
            case 2
                reso = 'Low noise';
            case 3
                reso = 'Balanced';
            case 4
                reso = 'High resolution';
            case 5
                reso = 'Maximum resolution';
        end
        % Detail settings.
        switch sets.isdetaildefault
            case 0
                detaildefault = 'Custom';
            case 1
                detaildefault = 'Default';
        end

        % Add settings to data structure.
        thisinfoline = {shortfilenames{imageind} allfilenames{imageind} sets.magnification sets.camerapixelsize seg targetch rotmixed rotdetail reso ... 
                        sets.lowlimit sets.highlimit sets.excludelow sets.excludehigh sets.lowguess sets.highguess ...
                        detaildefault sets.segmentlength sets.alpha sets.angleslack sets.convergence_threshold sets.maxiterations sets.sigma_prior_angle};
        % The text file needs duplicate info lines for each cell of an image.
        if strcmpi(filetype,'text')
            thisinfoline = repmat(thisinfoline,length(result),1);
        end
        infotowrite = [infotowrite; thisinfoline];

        % Add fibril data to the structure, leaving out the raw
        % distributions and angle/wavelength vectors.
        for cellind=1:length(result)
            if isstruct(result(cellind).fibrildata.orientation_mixed)
                temp1 = struct2cell(result(cellind).fibrildata.orientation_mixed);
                temp1 = temp1(4:end,:)';
            else
                temp1 = cell(oricols,1)';
            end
            if isstruct(result(cellind).fibrildata.distance_mixed)
                temp2 = struct2cell(result(cellind).fibrildata.distance_mixed);
                temp2 = temp2(3:end,:)';
            else
                temp2 = cell(wlcols,1)';
            end
            if isstruct(result(cellind).fibrildata.orientation_detail)                    
                temp3 = struct2cell(result(cellind).fibrildata.orientation_detail);
                temp3 = temp3(4:end,:)';
            else
                temp3 = cell(oricols,1)';
            end
            if isstruct(result(cellind).fibrildata.distance_detail)
                temp4 = struct2cell(result(cellind).fibrildata.distance_detail);
                temp4 = temp4(3:end,:)';
            else
                temp4 = cell(wlcols,1)';
            end

            % If there is no morphological data, leave it blank.
            if isempty(result(1).celldata)
                thisdataline = [shortfilenames{imageind} allfilenames{imageind} 1 'Mixed' temp1 temp2 'Detail' temp3 temp4 result(cellind).fibrildata.detailquality cell(1,cellcols)];
            % Otherwise add both data.
            else
                thisdataline = [shortfilenames{imageind} allfilenames{imageind} result(1).cellids(cellind) 'Mixed' temp1 temp2 'Detail' temp3 temp4 result(cellind).fibrildata.detailquality struct2cell(result(1).celldata(cellind))'];
            end
            datatowrite = [datatowrite; thisdataline];
        end
    end


    % Write an .xls spreadsheet.
    if strcmpi(filetype,'spreadsheet')
        set(waittext,'String','Writing spreadsheet file...');
        drawnow;
        datatowrite = [xlsheaderdata; datatowrite];
        exportsuccess = xlswrite(outputfile,datatowrite,1);
        infotowrite = [xlsheaderinfo; infotowrite];
        exportsuccess = xlswrite(outputfile,infotowrite,2);
        % Adjust column width.
        if exportsuccess == 1
            try
                autofitExcel(outputfile,2,'A:X');
                autofitExcel(outputfile,1,'A:AT');
            catch
            end
        end
    % Write text file.
    elseif strcmpi(filetype,'text')
        set(waittext,'String','Writing text file...');
        drawnow;
        txtheader = [xlsheaderdata xlsheaderinfo(3:end)];
        txtdata = [datatowrite infotowrite(:,3:end)];
        txttowrite = [txtheader; txtdata];
        tabletowrite = cell2table(txttowrite);
        writetable(tabletowrite,outputfile,'Delimiter','tab','WriteVariableNames',0);
    end


    % Kill waitbox.
    if emptyraw >= length(allfilenames)
        set(waittext,'String','No data to export!');
    else
        set(waittext,'String','Finished exporting data');
    end
    pause(1);
    set(waitdialog,'CloseRequestFcn','closereq');
    close(waitdialog);
    % Convert number of written bytes to 1 if write was succesfull.
    if exportsuccess > 0
        exportsuccess = 1;
    end

catch
    if ishandle(waitdialog)
        set(waitdialog,'CloseRequestFcn','closereq');
        close(waitdialog);
    end
end