function [gauss2d] = formGaussian2D(gridsize, mu, sigma, orientation)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Get the parameters.
Ny = gridsize(1);
Nx = gridsize(2);
y0 = mu(1);
x0 = mu(2);
sigma_y = sigma(1);
sigma_x = sigma(2);
theta = circ_ang2rad(orientation);

% Form the 2D Gaussian.
[X, Y] = meshgrid(1:Nx,1:Ny);
a = cos(theta)^2/2/sigma_x^2 + sin(theta)^2/2/sigma_y^2;
b = -sin(2*theta)/4/sigma_x^2 + sin(2*theta)/4/sigma_y^2;
c = sin(theta)^2/2/sigma_x^2 + cos(theta)^2/2/sigma_y^2;
gauss2d = exp(-(a*(X-x0).^2 + 2*b*(X-x0).*(Y-y0) + c*(Y-y0).^2));