function newgeneralsettings = generalsettingsPrompt(oldgeneralsettings)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

screensize = get(0,'ScreenSize');
defaultBackground = get(0,'defaultUicontrolBackgroundColor');
custompath = '';

% General components.
settingswindow = figure('Name','General settings','NumberTitle','Off','MenuBar','None', ...
    'WindowStyle','modal','CloseRequestFcn',@close_settingswindow,'units','pixels','OuterPosition', ...
    [0.35*screensize(3) 0.15*screensize(4) 0.3*screensize(3) 0.7*screensize(4)],'Color',defaultBackground);
okbutton = uicontrol(settingswindow,'Style','pushbutton','String','OK','Callback',@okbutton_Callback,'units','normalized','position',[0.55 0.02 0.2 0.05]);
cancelbutton = uicontrol(settingswindow,'Style','pushbutton','String','Cancel','Callback',@close_settingswindow,'units','normalized','position',[0.775 0.02 0.2 0.05]);

% Temporary file location selector.
temppathgroup = uibuttongroup('Parent',settingswindow,'Title','Location of temporary files:','SelectionChangeFcn',@temppathgroup_Callback,'BorderType','none','units','normalized','position',[0.05 0.65 0.9 0.3]);
defaultbutton = uicontrol(temppathgroup,'Style','radiobutton','String','Same as input images','units','normalized','position',[0.05 0.7 0.9 0.25]);
custombutton = uicontrol(temppathgroup,'Style','radiobutton','String','Custom location','units','normalized','position',[0.05 0.45 0.9 0.25]);
filetext = uicontrol(temppathgroup,'Style','text','String',custompath,'HorizontalAlignment','left','units','normalized','position',[0.05 0.25 0.7 0.1],'BackgroundColor',[1 1 1]);
browsebutton = uicontrol(temppathgroup,'Style','pushbutton','String','Browse','Callback',@browsebutton_Callback,'units','normalized','position',[0.8 0.15 0.15 0.2]);

% Unit selector.
unitgroup = uibuttongroup('Parent',settingswindow,'Title','Length units:','BorderType','none','units','normalized','position',[0.05 0.3 0.9 0.3]);
unitmenu = uicontrol(unitgroup,'Style','popupmenu','String','km|m|cm|mm|um|nm|pm|fm','units','normalized','position',[0.05 0.05 0.9 0.9],'BackgroundColor',[1 1 1]);

% Set defaults according to old settings.
% Temp path.
if strcmpi(oldgeneralsettings.temppath,'sameasimages')
    set(temppathgroup,'SelectedObject',defaultbutton);
    set(browsebutton,'Enable','off');
    set(filetext,'Enable','off');
    newgeneralsettings.temppath = oldgeneralsettings.temppath;
else
    set(temppathgroup,'SelectedObject',custombutton);
    set(browsebutton,'Enable','on');
    set(filetext,'Enable','on');
    custompath = oldgeneralsettings.temppath;
    set(filetext,'String',custompath);
    newgeneralsettings.temppath = custompath;
end
% Length unit.
switch oldgeneralsettings.dimensionfactor
    case 1e9 % Femtometers.
        set(unitmenu,'Value',8);
    case 1e6 % Picometers.
        set(unitmenu,'Value',7);
    case 1e3 % Nanometers.
        set(unitmenu,'Value',6);
    case 1 % Micrometers.
        set(unitmenu,'Value',5);
    case 1e-3 % Millimeters.
        set(unitmenu,'Value',4);
    case 1e-4 % Centimeters.
        set(unitmenu,'Value',3);
    case 1e-6 % Meters.
        set(unitmenu,'Value',2);
    case 1e-9 % Kilometers.
        set(unitmenu,'Value',1);
end
newgeneralsettings.dimensionfactor = oldgeneralsettings.dimensionfactor;

% Change radio button selection.
function temppathgroup_Callback(hObject,eventdata)
    selectedbutton = get(temppathgroup,'SelectedObject');
    if selectedbutton == defaultbutton
        set(browsebutton,'Enable','off');
        set(filetext,'Enable','off');
        newgeneralsettings.temppath = 'sameasimages';
    elseif selectedbutton == custombutton
        set(browsebutton,'Enable','on');
        set(filetext,'Enable','on');
        newgeneralsettings.temppath = custompath;
    end
end

% Browse for directory.
function browsebutton_Callback(hObject,eventdata)
    folder_name = uigetdir(custompath,'Select directory for temporary files');
    if isnumeric(folder_name) && folder_name == 0
        return;
    else
        custompath = folder_name;
        set(filetext,'String',custompath);
        newgeneralsettings.temppath = custompath;
    end
end

% If user presses cancel, old settings are retrieved before closing the
% window. If user presses ok, new settings will be returned.
function okbutton_Callback(hObject,eventdata)
    selectedobject = get(unitmenu,'Value');
    switch selectedobject
        case 8
            newgeneralsettings.dimensionfactor = 1e9;
            newgeneralsettings.unitsymbol = 'fm';
        case 7
            newgeneralsettings.dimensionfactor = 1e6;
            newgeneralsettings.unitsymbol = 'pm';                
        case 6
            newgeneralsettings.dimensionfactor = 1e3;
            newgeneralsettings.unitsymbol = 'nm';                
        case 5
            newgeneralsettings.dimensionfactor = 1;
            newgeneralsettings.unitsymbol = [char(0181),'m'];                
        case 4
            newgeneralsettings.dimensionfactor = 1e-3;
            newgeneralsettings.unitsymbol = 'mm';                
        case 3
            newgeneralsettings.dimensionfactor = 1e-4;
            newgeneralsettings.unitsymbol = 'cm';                
        case 2
            newgeneralsettings.dimensionfactor = 1e-6;
            newgeneralsettings.unitsymbol = 'm';                
        case 1
            newgeneralsettings.dimensionfactor = 1e-9;
            newgeneralsettings.unitsymbol = 'km';                
    end
    delete(settingswindow);
end

% Close the settings window if requested or wait for it.
function close_settingswindow(hObject,eventdata)
    newgeneralsettings = oldgeneralsettings;
    delete(settingswindow);
end
waitfor(settingswindow);

end

