function [PSD_1D] = getDirectedPSD(PSD_polar, theta_polar, principalangle, angleslack)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

if angleslack >= 90
    error('Error: angleslack must be less than 90 degrees!');
end
if principalangle >= 180
    error('Error: principalangle must be less than 180 degrees!');
end

% Make sure angular tolerance is not below angular resolution. This way at
% least one angle bin will be selected.
if angleslack <= (theta_polar(2) - theta_polar(1))
    angleslack = theta_polar(2) - theta_polar(1) + eps;
end

% Angle range falls out from the low end.
if principalangle - angleslack < 0
    angleoverwrap = abs(principalangle - angleslack);
    PSD_1D_first = PSD_polar(theta_polar < (principalangle + angleslack),:);
    PSD_1D_second = PSD_polar(theta_polar > (180 - angleoverwrap),:);
    PSD_1D = mean([PSD_1D_first; PSD_1D_second],1);
% Angle range falls out from the high end.
elseif principalangle + angleslack >= 180
    angleoverwrap = abs(principalangle + angleslack - 180);
    PSD_1D_first = PSD_polar(theta_polar > (principalangle - angleslack),:);
    PSD_1D_second = PSD_polar(theta_polar <= angleoverwrap,:);
    PSD_1D = mean([PSD_1D_first; PSD_1D_second],1);
% Angle range is within [0, 180].
else
    PSD_1D = mean(PSD_polar(abs(principalangle - theta_polar) < angleslack,:),1);
end