function plotSpectralROI(plothandle, spectrumtoplot, unitsymbol)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Show spectrum.
theta_deg = circ_rad2ang(spectrumtoplot.theta_polar);
[sizey,sizex] = size(spectrumtoplot.polar);
imagesc(spectrumtoplot.frequencies_polar,theta_deg,10*log10(spectrumtoplot.polar+eps),'Parent',plothandle);
% Use parula colormap if it exists for this MATLAB version.
try
    colormap parula;
% Revert to default colormap if necessary.
catch
    colormap default;
end
hold on;

% Show mixed ROI boundaries.
for k=1:length(spectrumtoplot.mixedROIboundary)
    bounds = spectrumtoplot.mixedROIboundary{k};
    bounds(:,1) = (max(theta_deg)/sizey)*bounds(:,1);
    bounds(:,2) = (max(spectrumtoplot.frequencies_polar)/sizex)*bounds(:,2);
    plot(plothandle,bounds(:,2),bounds(:,1),'k','LineWidth',2);
end
% Show detail ROI boundaries.
for k=1:length(spectrumtoplot.detailROIboundary)
    bounds = spectrumtoplot.detailROIboundary{k};
    bounds(:,1) = (max(theta_deg)/sizey)*bounds(:,1);
    bounds(:,2) = (max(spectrumtoplot.frequencies_polar)/sizex)*bounds(:,2);
    plot(plothandle,bounds(:,2),bounds(:,1),'r','LineWidth',2);
end

ylabel(plothandle,'Angle (deg)');
xlabel(plothandle,{'Spatial frequency';['(1/',unitsymbol,')']});

