function plotWavelengthDistribution(plothandle, wavelengthdata, unitsymbol)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% If there is no data to plot, show an empty plot.
if isempty(wavelengthdata)
    plot(plothandle,1:50,zeros(1,50));
    axis(plothandle,[0 50 0 1]);
    xlabel(plothandle,['Wavelength (',unitsymbol,')']);
    return;
end

% Plot data.
axhandle = plot(plothandle, wavelengthdata.polar, wavelengthdata.distribution,'k');
set(axhandle,'linewidth',2);
xlabel(plothandle,['Wavelength (',unitsymbol,')']);

% Set the default limits of the plot view.
leftbound = wavelengthdata.mean-6*wavelengthdata.std;
rightbound = wavelengthdata.mean+6*wavelengthdata.std;
if leftbound < 0
    leftbound = 0;
end
if rightbound > max(wavelengthdata.polar)
    rightbound = max(wavelengthdata.polar);
end
if leftbound ~= rightbound && ~isnan(leftbound) && ~isnan(rightbound)
    axis(plothandle,[leftbound rightbound min(wavelengthdata.distribution) max(wavelengthdata.distribution)]);
end