function [mu, medi, mo, vari, stdev, skew, kurt] = statsFromPDF(frequency, values, robustness)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Find nonzero values.
L = length(frequency);
nonzeros = find(frequency~=0);
N = length(nonzeros);
values_nonzero = values(nonzeros);
values_nonzero = values_nonzero(:);
% Normalize pdf estimate.
frequency_norm = frequency(nonzeros)/sum(frequency(nonzeros));
frequency_norm = frequency_norm(:);

% If robustness used, remove the extreme quantiles according to given
% percentage. NOTE: this is not used in CytoSpectre.
if robustness > 0
    robustness = robustness/100;
    cumulativesum = cumsum(frequency_norm);
    nonzeros = find(robustness/2 < cumulativesum & cumulativesum < 1-robustness/2);
    frequency_norm = frequency_norm(nonzeros);
    frequency_norm = frequency_norm/sum(frequency_norm);
    values_nonzero = values_nonzero(nonzeros);
end

% Mean.
if L > 0
    mu = sum(values_nonzero.*frequency_norm);
else
    mu = NaN;
end

% Median.
if L > 0
    medianindex = find(cumsum(frequency_norm) >= 0.5*sum(frequency_norm),1,'first');
    medi = values_nonzero(medianindex);
else
    medi = NaN;
end

% Mode.
if L > 0
    maxindex = find(frequency_norm == max(frequency_norm),1,'first');
    mo = values_nonzero(maxindex);
else
    mo = NaN;
end

% Variance and standard deviation.
if N > 1
    devifrommean = values_nonzero - mu;
    vari = sum(devifrommean.*devifrommean.*frequency_norm);
    stdev = sqrt(vari);
else
    vari = NaN;
    stdev = NaN;
end

% Skewness.
if N > 1
    skew = sum(devifrommean.*devifrommean.*devifrommean.*frequency_norm)/(stdev*vari);
else
    skew = NaN; 
end

% Kurtosis.
if N > 1
    kurt = sum(devifrommean.*devifrommean.*devifrommean.*devifrommean.*frequency_norm)/(vari*vari);
else
    kurt = NaN; 
end
